C
C  WLUTIL
C
C  WIMS LIBRARY UTILITY FOR NRSC SYSTEM OR OTHER (RESONANCE PARAMETERS)
C  [assumed standard groups WIMS library (69(14,27,42)]
C
C  AUTHOR: FRANCISCO LESZCZYNSKI-INGENIERIA NUCLEAR- 1999
C                                CENTRO ATOMICO BARILOCHE
C
C  OPTION 1: GENERATES A NEW WIMS LIBRARY WITH NEW LAMBDA FACTORS FOR ONE OR 
C            MORE ISOTOPES FROM INPUT (ON WLUTIL.DAT)
C
C  OPTION 2: GENERATES OUTPUT (ON RI.DAT) FROM WIMS LIBRARY DATA OF
C            RI VS.XSb FOR A RESONANT ISOTOPE 
C      
C  OPTION 3: GENERATES A NEW WIMS LIBRARY WITH ALL DATA OF RI VS.XSb FOR A 
C            RESONANT ISOTOPE REPLACED FROM INPUT (ON RI.OUT)
C
C  OPTION ... (CAN BE ADDED ANY OTHER OPTION ACCORDING TO USER REQUESTS)
C
C--------------------------------------------------------------------------
C   Input files: WLUTIL.DAT  input data 
C                WIMS1.LIB (INPUT WIMS LIBRARY in ascii-WILLIE format) 
C      option 3: RI.OUT (RI vs XSb-RMET21 output format-a set of data 
C                                                       for each temperature
C                                                       WITH 13 GROUPS
C--------------------------------------------------------------------------
C   Output files: 
C      option 1,3: WIMS2.LIB (OUTPUT WIMS LIBRARY in ascii-WILLIE format)
C      option 2:   RI.DAT (RI vs XSb)
C
C--------------------------------------------------------------------------
C   INPUT data:
C
C   card 1 (I5): ioptio  = 1 -> option 1 (replace lambdas);
C                        = 2 -> option 2 (extract RI vs.XSb for one isotope);
C                        = 3 -> option 3 (replace RI vs.XSb for one isotope)
C
C   if ioptio=1:
C            card 2 (I5): niss (=# of isotopes to be changed its lambda)
C            card 3 (I5): idss(i) -> WIMS identifier of isotope
C            card 4 (F11.0): alss(i) -> new lambda for isotope
C            Repeat cards 3,4 for the niss isotopes
C
C   if ioptio=2:
C            card 2 (I5): isoto -> WIMS identifier of isotope (integer)
C            card 3 (i5): iesnof (=0->no fission data;
C                                 =1->fission resonances included)
C
C   if ioptio=3:
C            card 2 (I5): isoto -> WIMS identifier of isotope (integer)
C            card 3 (4I5): numtem -> number of temperatures
C                          numxsb -> number of XSb
C                          iesnof (=0->no fission data;
C                                  =1->fission resonances included)
C            card 4 (6E12.5): treso(i),i=1,numtem-> temperatures (K)
C       
C***********************************************************************
C***********************************************************************
C
      DIMENSION IDENT(200),NDAT1(200),NTEMP(200),MTF(200,200)
      DIMENSION EG(200),CHI(200),AW(200),A1(200),A2(200),A(20000)
      DIMENSION TEMP(30),TRES(20),SIGB(20),RI(400)
      DIMENSION YLD(200,200),SPOT(200,100),XISPDU(200,100)
      DIMENSION ALSPOT(200,100),AXWP1(4,200,200)
      DIMENSION IDSS(150)
      DIMENSION ALSS(150)
      DIMENSION XSB(20),RA(10,20,13),RF(10,20,13),TRESO(10)
C
C***********************************************************************
       OPEN(UNIT=5,FILE='WLUTIL.DAT',FORM='FORMATTED',STATUS='OLD')
C
        READ(5,'(I5)')IOPTIO
        IF(IOPTIO.EQ.1)THEN
         READ(5,'(I5)')NISS
         DO I=1,NISS
          READ(5,'(I5)')IDSS(I)
          READ(5,'(F11.0)')ALSS(I)
         ENDDO
        ELSEIF(IOPTIO.EQ.2)THEN
         READ(5,'(I5)')ISOTO
         READ(5,'(I5)')IESNOF
        ELSEIF(IOPTIO.EQ.3)THEN
         READ(5,'(I5)')ISOTO
         READ(5,'(3I5)')NUMTEM,NUMXSB,IESNOF
         READ(5,'(6E12.5)')(TRESO(I),I=1,NUMTEM)
         OPEN(UNIT=3,FILE='RI.OUT',FORM='FORMATTED',STATUS='OLD')
         DO IT=1,NUMTEM
          DO IXSB=1,NUMXSB       
           DO IGR=1,13
            IF(IESNOF.EQ.1)THEN
             READ(3,'(I5,1P3E12.5)')IGRR,XSB(IXSB),RA(IT,IXSB,IGR),
     1                                             RF(IT,IXSB,IGR)
            ELSE
             READ(3,'(I5,1P3E12.5)')IGRR,XSB(IXSB),RA(IT,IXSB,IGR),DUMMY
            ENDIF
           ENDDO
          ENDDO
         ENDDO 
         CLOSE(3)
        ENDIF
        CLOSE(5)
C
       OPEN(UNIT=1,FILE='WIMS1.LIB',FORM='FORMATTED',STATUS='OLD')
       IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
        OPEN(UNIT=2,FILE='WIMS2.LIB',FORM='FORMATTED',STATUS='NEW')
       ENDIF
C
C 1) GENERAL DATA
C
        READ(1,'(5I15)')NEL,NOG,NCHI,NFG,NRG,NTG,NFIS,NFP
        IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
         WRITE(2,'(5I15)')NEL,NOG,NCHI,NFG,NRG,NTG,NFIS,NFP
        ENDIF
C
        NG12=NFG+NRG
        READ(1,'(5I15)')(IDENT(I),I=1,NEL)
        IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
         WRITE(2,'(5I15)')(IDENT(I),I=1,NEL)
        ENDIF
C
        READ(1,'(5E15.8)')(EG(I),I=1,NOG),WCERO
        IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
         WRITE(2,'(1P,5E15.8)')(EG(I),I=1,NOG),WCERO
        ENDIF
C
        READ(1,'(5E15.8)')(CHI(K),K=1,NCHI)
        IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
         WRITE(2,'(1P,5E15.8)')(CHI(K),K=1,NCHI)
        ENDIF
C
       DO I=1,NEL
        READ(1,'(I15/(3(E15.8,I6)))')NDAT1(I),
     1                               (YLD(I,J),MTF(I,J),J=1,NDAT1(I)/2)
        IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
         WRITE(2,'(I15/(3(1PE15.8,I6)))')NDAT1(I),
     1                               (YLD(I,J),MTF(I,J),J=1,NDAT1(I)/2)
        ENDIF
       ENDDO
C
        READ(1,'(I15)') ifilemar
        IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
         WRITE(2,'(I15)') ifilemar
        ENDIF
C
C 2) XS
C
       DO I=1,NEL
         READ(1,'(I6,E15.8,5I6)')ID,AW(I),IZ,IDCOD,NTEMP(I),NRR,NN
         IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
         WRITE(2,'(I6,1PE15.8,5I6)')ID,AW(I),IZ,IDCOD,NTEMP(I),NRR,NN
         ENDIF
C
C  EPITHERMAL GROUPS
C
         READ(1,'(5E15.8)')    (SPOT(I,IG),IG=1,NRG),
     1                         (XISPDU(I,IG),IG=1,NRG),
     2                         (A1(IG),IG=1,NG12),
     3                         (A2(IG),IG=1,NG12),
     4                         (XQ,IG=1,NRG),
     5                         (ALSPOT(I,IG),IG=1,NRG)
C
      IF(IOPTIO.EQ.1)THEN
       DO KK1=1,NISS
        IF(ID.EQ.IDSS(KK1))THEN
         DO KK2=1,NRG
          ALSPOT(I,KK2)=ALSS(KK1)
         ENDDO
        ENDIF
       ENDDO
      ENDIF
C
        IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
         WRITE(2,'(1P,5E15.8)') (SPOT(I,IG),IG=1,NRG),
     1                          (XISPDU(I,IG),IG=1,NRG),
     2                          (A1(IG),IG=1,NG12),
     3                          (A2(IG),IG=1,NG12),
     4                          (XQ,IG=1,NRG),
     5                          (ALSPOT(I,IG),IG=1,NRG)
        ENDIF
C
        IF(IDCOD.GT.1)THEN      
C  XS-FIS,NU*XS-FIS
          READ(1,'(5E15.8)')(A1(IG),IG=1,NG12),(A2(IG),IG=1,NG12)
          IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
           WRITE(2,'(1P,5E15.8)')(A1(IG),IG=1,NG12),(A2(IG),IG=1,NG12)
          ENDIF
        ENDIF
C  MATRIX
         READ(1,'(I15/(5E15.8))')NDAT2,(A(IDA),IDA=1,NDAT2)
         IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
          WRITE(2,'(I15/(1P,5E15.8))')NDAT2,(A(IDA),IDA=1,NDAT2)
         ENDIF
C  
C  THERMAL GROUPS
C
         READ(1,'(5E15.8)')(TEMP(IT),IT=1,NTEMP(I))
         IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
          WRITE(2,'(1P,5E15.8)')(TEMP(IT),IT=1,NTEMP(I))
         ENDIF
         DO IT1=1,NTEMP(I)
          READ(1,'(5E15.8)')(A1(IG2),IG2=1,NTG),(A2(IG2),IG2=1,NTG)
          IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
          WRITE(2,'(1P,5E15.8)')(A1(IG2),IG2=1,NTG),(A2(IG2),IG2=1,NTG)
          ENDIF
         IF(IDCOD.GT.1)THEN
           READ(1,'(5E15.8)')(A1(IG2),IG2=1,NTG),(A2(IG2),IG2=1,NTG)
           IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
          WRITE(2,'(1P,5E15.8)')(A1(IG2),IG2=1,NTG),(A2(IG2),IG2=1,NTG)
           ENDIF
         ENDIF
C  MATRIX
          READ(1,'(I15/(5E15.8))')NDAT3,(A(ID3),ID3=1,NDAT3)
          IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
           WRITE(2,'(I15/(1P,5E15.8))')NDAT3,(A(ID3),ID3=1,NDAT3)
          ENDIF
        ENDDO
         READ(1,'(I15)')ifilemar
         IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
          WRITE(2,'(I15)')ifilemar
         ENDIF
       ENDDO
C
C 3) RI
C
       DO IGR=1,NRG
        IPASO=0
 100    READ(1,'(I15)')M1M2
        IF(M1M2.EQ.ifilemar)GOTO 200
         READ(1,'(E15.8,2I6/(5E15.8))') RIDD,M1,M2,
     1              (TRES(IM1),IM1=1,M1),(SIGB(JM),JM=1,M2),
     2              (RI(IM),IM=1,M1M2)
C
        IF(IOPTIO.EQ.2.OR.IOPTIO.EQ.3)THEN
         IF(NINT(RIDD).EQ.ISOTO)THEN
          IF(IOPTIO.EQ.2)THEN
           IF(IGR.EQ.1.AND.IPASO.EQ.0)THEN
            M111=M1
            M222=M2
            DO IXSB=1,M2
             XSB(IXSB)=SIGB(IXSB)
            ENDDO
            DO IT=1,M1
             TRESO(IT)=TRES(IT)
            ENDDO
           ENDIF
           DO IT=1,M1
            DO IXSB=1,M2
             IXXSB=IXSB+M2*(IT-1)
             IF(IESNOF.EQ.1)THEN
              IF(IPASO.EQ.0)THEN
               RA(IT,IXSB,IGR)=RI(IXXSB) 
              ELSE
               RF(IT,IXSB,IGR)=RI(IXXSB)
              ENDIF
             ELSEIF(IESNOF.EQ.0)THEN
              RA(IT,IXSB,IGR)=RI(IXXSB) 
             ENDIF
            ENDDO
           ENDDO
           IF(IPASO.EQ.0)IPASO=1
          ELSEIF(IOPTIO.EQ.3)THEN
           M1=NUMTEM
           M2=NUMXSB
           M1M2=M1*M2
           DO IT=1,M1
            TRES(IT)=TRESO(IT)
            DO IXSB=1,M2
             SIGB(IXSB)=XSB(IXSB)
             IXXSB=IXSB+M2*(IT-1)
             IF(IESNOF.EQ.1)THEN
              IF(IPASO.EQ.0)THEN
               RI(IXXSB)=RA(IT,IXSB,IGR) 
              ELSE
               RI(IXXSB)=RF(IT,IXSB,IGR)
              ENDIF
             ELSEIF(IESNOF.EQ.0)THEN
              RI(IXXSB)=RA(IT,IXSB,IGR) 
             ENDIF
            ENDDO
           ENDDO
           IF(IPASO.EQ.0)IPASO=1
          ENDIF
         ENDIF
        ENDIF
C
         IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
          WRITE(2,'(I15/1PE15.8,2I6/(1P,5E15.8))') M1M2, 
     1                                        RIDD,M1,M2,
     2              (TRES(IM1),IM1=1,M1),(SIGB(JM),JM=1,M2),
     3              (RI(IM),IM=1,M1M2)
         ENDIF
         GOTO 100
 200     IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
          WRITE(2,'(I15)')ifilemar
         ENDIF
        ENDDO
C
       IF(IOPTIO.EQ.2)THEN
        OPEN(UNIT=6,FILE='RI.DAT',FORM='FORMATTED',STATUS='NEW')
        DUMMY=0.0
        WRITE(6,'("RI VS.XSb-ISOTOPE ",I5)')ISOTO
        WRITE(6,'("NUMBER OF TEMPERATURES=",I5)')M111
        WRITE(6,'("NUMBER OF XSb=",I5)')M222
        WRITE(6,'("TEMPERATURES=",1P6E12.5)')(TRESO(III),III=1,M111)
        WRITE(6,'("DO NOT FORGET TO ADD A FIRST EXTRA GROUP WITH DUMMY D
     1ATA FOR INPUT TO RMET21")')
        WRITE(6,'("DELETE UNTIL THIS RECORD FOR INPUT TO RMET21")')
        DO IT=1,M111
         DO IXSB=1,M222
          DO IGR=1,13
           IF(IESNOF.EQ.1)THEN
            WRITE(6,'(I5,1P3E12.5)')IGR,XSB(IXSB),RA(IT,IXSB,IGR),
     1                                            RF(IT,IXSB,IGR)
           ELSE
            WRITE(6,'(I5,1P3E12.5)')IGR,XSB(IXSB),RA(IT,IXSB,IGR),DUMMY
           ENDIF
          ENDDO
         ENDDO
        ENDDO
        CLOSE(6)
       ENDIF
C
C 4) P1
C
       DO IWP1=1,4
        DO IWGP1=1,NOG
          READ(1,'(5E15.8)')(AXWP1(IWP1,IWGP1,IWGP2),IWGP2=1,NOG)
          IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
           WRITE(2,'(1P,5E15.8)')(AXWP1(IWP1,IWGP1,IWGP2),IWGP2=1,NOG)
          ENDIF
        ENDDO
       ENDDO
C
        READ(1,'(I15)') ifilemar
        IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
         WRITE(2,'(I15)') ifilemar
        ENDIF
C
       CLOSE(1)
       IF(IOPTIO.EQ.1.OR.IOPTIO.EQ.3)THEN
        CLOSE(2)
       ENDIF
C***********************************************************************
C All processing completed
C
      STOP 'Completed'
C
      END
