C     ***************                                                   
C     *             *                                                   
C     * G E X S C O *                                                   
C     *             *                                                   
C     ***************                                                 
C                                                                       
C     Program for generation of punctual cross sections              
C     on fine and constant lethargy intervals                        
C     from the linearly interpolable cross sections in ENDF/B format 
C     (formatted output of Cullen-SIGMA1 code or NJOY-BROADR)       
C                                                                       
C     Author: Francisco Leszczynski                             
C             Centro Atomico Bariloche                                   
C             8400 S.C.de Bariloche, RN
C             ARGENTINA
C                                                                       
C     First version: February 1985
C     This version:  March    1999
C--------------------------------------------------------------------------------
c                                                                        gexsco.1    
c     program gexsco                                                     gexsco.2    
c                                                                        gexsco.3    
c     ****************************************************************** gexsco.4    
c     *                                                                * gexsco.5    
c     * Program for generation of punctual cross sections              * gexsco.6       
c     * on fine and constant lethargy intervals                        * gexsco.7
c     * from the linearly interpolable cross sections in ENDF/B format * gexsco.8
c     * (formatted output of Cullen-SIGMA1 code or NJOY-BROADR)        * gexsco.9
c     * The generated file is used as input on RMET21 program, that    * gexsco.10
c     * calculate neutron flux on the energy region of neutron         * gexsco.11    
c     * resonances of heavy isotopes for homogeneous systems and       * gexsco.12   
c     * cylindrical pin cells, on a very fine energy mesh of constant  * gexsco.13
c     * lethargy.                                                      * gexsco.14
c     *                                                                * gexsco.15   
c     *    --- input ---                                               * gexsco.16   
c     *                                                                * gexsco.17   
c     * file        contents                                           * gexsco.18   
c     * GEXSCO.DAT  user input file                                    * gexsco.19   
c     * XS.ASC      cross sections input of the resonant isotope       * gexsco.20  
c     *             (formatted output of Cullen-SIGMA1 code            * gexsco.21
c     *              or NJOY-BROADR)                                   * gexsco.22
c     *                                                                * gexsco.23
c     *                                                                * gexsco.24   
c     *    --- output ---                                              * gexsco.25   
c     *                                                                * gexsco.26   
c     * file        contents                                           * gexsco.27   
c     * GEXSCO.OUT  control output file                                * gexsco.28   
c     * XS1         cross sections output of the resonant isotope      * gexsco.29  
c     *             (for input to RMET21 program)                      * gexsco.30
c     *                                                                * gexsco.31   
c     *    --- auxiliary files ---                                     * gexsco.32   
c     *                                                                * gexsco.33   
c     * XAUX1,XAUX2,XAUX3 These files must be deleted after finished   * gexsco.34   
c     *                   each GEXSCO run                              * gexsco.35   
c     *                                                                * gexsco.36   
c     *---input specifications in file GEXSCO.DAT----------------------* gexsco.37   
c     *                                                                * gexsco.38   
c     * card 1 (format:3I7)                                            * gexsco.39   
c     *    npft     total number of lethargy points                    * gexsco.40
c     *    ni       # of lethargy points on each intermediate group    * gexsco.41
c     *    ifisio   fission option                                     * gexsco.42   
c     *               0  no include fission cross sections             * gexsco.43
c     *               1  include fission cross sections                * gexsco.44
c     *                                                                * gexsco.45
c     * card 2 (format:2F11.0)                                         * gexsco.46   
c     *    emin     min energy (eV)                                    * gexsco.47
c     *    emax     max energy (eV)                                    * gexsco.48
c     *                                                                * gexsco.49
c     *  WARNING !!!                                                   * gexsco.50
c     *  No-exhaustive consistency checks are automatically made on    * gexsco.51
c     *  the input file. It is responsability of the user to check all * gexsco.52  
c     *  the entries for each case.                                    * gexsco.53
c     *                                                                * gexsco.54
c     ****************************************************************** gexsco.55
c                                                                        gexsco.56
C--------------------------------------------------------------------------------
      DIMENSION EV(150000),XV(150000)
      DIMENSION XS1(200000),XS2(200000),XS18(200000),XSA(200000)
      DIMENSION MMT(3)
      CHARACTER*70 AA
      REAL*4 DTIME,TARRAY(2),CTIEM,FTIEM
C
  1   FORMAT(3I7)                                                     
  2   FORMAT(2F11.0)                                               
  3   FORMAT(A70,I2,I3)
  4   FORMAT(I11)
  5   FORMAT(3(F11.0,E11.5))
  6   FORMAT(1P4E15.8)
  7   FORMAT('WARNING.THERE ARE ',I7,' POINTS ON E=',1PE15.8)
  8   FORMAT('DELTA U= ',1PE15.8)
  9   FORMAT('TOTAL NUMBER OF LETHARGY POINTS=',I7/
     1'NUMBER OF LETHARGY POINTS ON EACH INTERMEDIATE GROUP=',I7/
     2'INCLUDE FISSION CROSS SECTIONS(0:NO/1:YES)=',I7/)
100   FORMAT('MIN ENERGY (EV)=',1PE12.5/
     1'MAX ENERGY (EV)=',1PE12.5/)
110   FORMAT(' NUMBER OF POINTS FOR XStotal ON XS.ASC=',I6)
120   FORMAT(' NUMBER OF POINTS FOR XSscat. ON XS.ASC=',I6)
130   FORMAT(' NUMBER OF POINTS FOR XSfiss. ON XS.ASC=',I6)
140   FORMAT(' ALL DONE')
C
      CTIEM=DTIME(TARRAY) 
      IE=5                                                              
      IS=4                                                              
      IXSE=7                                                           
      IXSS=8                                                            
      IXAU1=9
      IXAU2=10
      IXAU3=11
      OPEN(UNIT=5,FILE='GEXSCO.DAT',FORM='FORMATTED',STATUS='OLD')
      OPEN(UNIT=4,FILE='GEXSCO.OUT',FORM='FORMATTED',STATUS='NEW')
      OPEN(UNIT=7,FILE='XS.ASC',STATUS='OLD')
      OPEN(UNIT=8,FILE='XS1',FORM='UNFORMATTED',STATUS='NEW')
      OPEN(UNIT=9,FILE='XAUX1',FORM='UNFORMATTED',STATUS='NEW')
      OPEN(UNIT=10,FILE='XAUX2',FORM='UNFORMATTED',STATUS='NEW')
      OPEN(UNIT=11,FILE='XAUX3',FORM='UNFORMATTED',STATUS='NEW')      
      CALL TIMEDATE           
C                                                                       
C     read data                                                  
C                                                                       
      READ(IE,1)NPFT,NI,IFISIO                                        
      WRITE(IS,9)NPFT,NI,IFISIO                                         
      PRINT *,"INPUT 1: ",NPFT,NI,IFISIO   
C                                                                       
      READ(IE,2)EMIN,EMAX                                              
      WRITE(IS,100)EMIN,EMAX                                              
      PRINT *,"INPUT 2: ",EMIN,EMAX                                             
C
      DU1=ALOG(EMAX/EMIN)
      DU=DU1/FLOAT(NPFT)
      WRITE(IS,8)DU
      PRINT *,"DELTA U: ",DU
C
C     write head data on XS1
C 
      WRITE(IXSS)NPFT,IFISIO,NI
      WRITE(IXSS)DU
      WRITE(IXSS)EMAX
C
      MMT(1)=1
      MMT(2)=2
      MMT(3)=18
C
C     Position of XS.ASC file at the begining of MF=3
C
  10  READ(IXSE,3)AA,MF,MT           
      IF(MF.LT.3)GOTO 10
C
      NXS=0
C
  20  NXS=NXS+1
C
C     read EV(),XV() from XS.ASC for each reaction MT=1,2
C     (and 18 if NFISIO=1)
C
      IF(NXS.GT.1)THEN
  30   READ(IXSE,3)AA,MF,MT           
       IF(MT.LT.MMT(NXS))GOTO 30 
      ENDIF
C
      READ(IXSE,3)AA,MF,MT
      READ(IXSE,4)NPEX
      IF(NXS.EQ.1)THEN   
       WRITE(IS,110)NPEX
       PRINT *,"NUMBER OF POINTS FOR XStotal ON XS.ASC=",NPEX
      ELSEIF(NXS.EQ.2)THEN   
       WRITE(IS,120)NPEX
       PRINT *,"NUMBER OF POINTS FOR XSscat. ON XS.ASC=",NPEX
      ELSEIF(NXS.EQ.3)THEN   
       WRITE(IS,130)NPEX
       PRINT *,"NUMBER OF POINTS FOR XSfiss. ON XS.ASC=",NPEX
      ENDIF  
      READ(IXSE,5)(EV(I),XV(I),I=NPEX,1,-1)
C
      J=NPEX+1
  40  J=J-1
      IF(EV(J).LT.EMAX)GOTO 40
      J1=J
C
      E1=EMAX
      J=J1
      NIT=0
      I=0
  45  DE=0
      SAL=0
      XX=0
      I=I+1
      E2=E1*EXP(-DU)
      NK=0
  50  IF(EV(J+1).LE.E2)THEN
       EK=E2
       SAL=1
      ELSE
       EK=EV(J+1)
       NK=NK+1
      ENDIF
      EM=(E1+EK)/2.0
      DEM=E1-EK
      DE=DE+DEM
      A=(XV(J)-XV(J+1))/(EV(J)-EV(J+1))
      B=XV(J)-A*EV(J)
      X=A*EM+B
      XX=XX+X*DEM
      E1=EK
      J=J+1
      IF(SAL.EQ.0)GOTO 50
C
      J=J-1
      XS1(I)=XX/DE 
      IF(XS1(I).LT.0.0)XS1(I)=0.0
      IF(NK.GT.1)THEN
       WRITE(IS,7)NK,E2
       PRINT *,"WARNING.THERE ARE ",NK," POINTS ON E=",E2
      ENDIF
      IF(I.EQ.NI)THEN
       IF(NXS.EQ.1)THEN
        IXAU=IXAU1
       ELSEIF(NXS.EQ.2)THEN
        IXAU=IXAU2
       ELSEIF(NXS.EQ.3)THEN
        IXAU=IXAU3
       ENDIF
       WRITE(IXAU)(XS1(I),I=1,NI)
       NIT=NIT+NI
       I=0
      ENDIF
C
      IF(NIT.LT.NPFT)GOTO 45
C
      IF(NXS.EQ.1)GOTO 20
      IF(NXS.EQ.2.AND.IFISIO.EQ.1)GOTO 20
C
C     ********************************************************
C
      REWIND(IXAU1)
      REWIND(IXAU2)
      REWIND(IXAU3)
C
      NN=NPFT/NI
      E1=EMAX
      DO J=1,NN
       READ(IXAU1)(XS1(I),I=1,NI)
       READ(IXAU2)(XS2(I),I=1,NI)
       IF(IFISIO.EQ.1)THEN
        READ(IXAU3)(XS18(I),I=1,NI)
       ELSE
        DO I=1,NI
         XS18(I)=0.0
        ENDDO
       ENDIF
C
       DO I=1,NI
        XSA(I)=XS1(I)-XS2(I)
        IF(XSA(I).LT.0.0)XSA(I)=0.0
        IF(XSA(I).LT.XS18(I))XSA(I)=XS18(I)
       ENDDO
C
C      write xs data on XS1
C 
       WRITE(IXSS)(XSA(J1),J1=1,NI)
       WRITE(IXSS)(XS2(J3),J3=1,NI)                             
       IF(IFISIO.EQ.1)WRITE(IXSS)(XS18(J2),J2=1,NI)
C
       E1=E1*EXP(-DU*FLOAT(NI))
      ENDDO
      CALL TIMEDATE
      WRITE(IS,140)
      PRINT *,"ALL DONE"
      FTIEM=DTIME(TARRAY)
      WRITE(IS,'("Total cpu time(min)=",F10.3)')FTIEM/60.0
C
      CLOSE(5)
      CLOSE(4)
      CLOSE(7)
      CLOSE(8)
      CLOSE(9)
      CLOSE(10)
      CLOSE(11)
C
      STOP
      END
C-------------------------------------------------------------
      SUBROUTINE TIMEDATE    
       character * 11, time_result, date_result
C       call system("cls") 
       call time (time_result)
       WRITE(4,'("The current time is: ",A11)')time_result
       PRINT *,"The current time is: ",time_result
       call date (date_result)
       WRITE(4,'("The current date is: ",A11)')date_result
       PRINT *,"The current date is: ",date_result
      RETURN
      END

